/*	AMX Mod X script.
    
	Preset KillStreak plugin

	(c) Copyright 2007, Simon Logic 'slspam@land.ru'
	This file is provided AS IS (no warranties).

	Info:
		UT-like killing spree announcer messages with custom preset support. 
		You can	make your own killing streaks without modifying 
		the source code. Now there is no need to beg AMX/X scripters to add 
		your favourite (e.g. sexy) sounds & sequences into plugin :)

	Features:
		* up to 10 custom levels of killing streak
		* unlimited killing streak levels support by using unified announce
		* presets supported (see 'amxmodx/config/preset_killstreak.ini' file)
		* per map presets supported
		* per map prefix presets supported
		* mp_freeforall & mp_teamplay cvars supported to distinguish teamplay
		(thus CSDM FFA & HLDM teamplay supported!)
		* default messages are exactly the same as seen in UT (except announce
		message when streaker killed himself - it was normalized)
		* log streak messages into mod's log when mp_logdetail = 3 
		(this lets you introduce new positive bonuses in PsychoStats)
		* and more...
	
	Requirements:
		* any mod
		* AMX/X 1.7x or higher

	New cvars:
		* pks_kills_per_streak <num> (default=5)
			kills per one streak level
		* pks_kills_basis <num> (default=0)
			basic number of kills to count streak levels from; negative 
			numbers are possible; e.g. value is -3, then player should
			kill (5 - 3) = 2 players to get 1st streak level
		* pks_announce_extra_levels <0|1> (default=1)
			when a player achieved max streak level other levels will be 
			announced in a unified manner with number of frags:
			0 - do not announce extra levels
			1 - announce extra levels till player death
		* [CS] pks_reset_on_restart <0|1> (default=0)
			reset counters on new round

	Changelog:

	v1.1.3 [2007-08-05]
	! sounds were precached for built-in preset only
	! when player being on a killing spree was killed by worldspawn a sound
	for killer could be played on everyone
	+ now sound for others is played on a victim also
	! when player being on a killing spree killed himself a sound for killer
	(e.g. woohoo2.wav) was played on him
	v1,1.2 [2007-07-26]
	+ added 'pks_kills_basis' cvar
	v1.1.1 [2007-07-17]
	! selective disabling of killing streak levels is truly supported now
	v1.1.0 [2007-07-07]
	+ added 'pks_announce_extra_levels' cvar
	* changed .ini format slighlty: added one more service line
	* changed logging string to unified string when mp_logdetail = 3
	v1.0.1 [2007-07-04]
	! sounds for other players were not precached (was a copy-paste typo)
	v1.0.0 [2007-07-03]
	* first release
*/

#include <amxmodx>
#include <amxmisc>

#pragma tabsize 4
#pragma ctrlchar '\'

#define MY_PLUGIN_NAME    "Preset KillStreak"
#define MY_PLUGIN_VERSION "1.1.3"
#define MY_PLUGIN_AUTHOR  "Simon Logic"

#define MAX_CLIENTS 32
#define MAX_KILLSTREAK_LEVELS 10
#define MAX_SND_LENGTH 64
#define MAX_MSG_LENGTH 128
#define AUX_LEVELS 2
	// don't change it until you realize fully your action

#define _PRECACHE
//#define _DEBUG
//#define _ASSERT

new const 
	g_sConfFileName[] = "preset_killstreak.ini",
	g_sAuxConfDir[] = "preset_killstreak",
	g_sSoundDir[] = "sound",
	g_sSplitter[] = "::"

// default built-in preset
new const g_sDefPreset[][] = {
	"%s is on a killing spree!::misc/killingspree.wav::misc/strangecrap.wav",
	"%s is on a rampage!::misc/rampage.wav::misc/strangecrap.wav",
	"%s is dominating!::misc/dominating.wav::misc/strangecrap.wav",
	"%s is unstoppable!::misc/unstoppable.wav::misc/strangecrap.wav",
	"%s is Godlike!::misc/godlike.wav::misc/strangecrap.wav",
	"%s is on a killing spree with %d frags!::misc/killingspree.wav::misc/strangecrap.wav", // extra level announce
	"%s's killing spree was ended by %s::misc/woohoo2.wav" // over
}

new g_sSndStreakKiller[MAX_KILLSTREAK_LEVELS+AUX_LEVELS][MAX_SND_LENGTH+1]
new g_sSndStreakOthers[MAX_KILLSTREAK_LEVELS+AUX_LEVELS][MAX_SND_LENGTH+1]
new g_sMsgStreak[MAX_KILLSTREAK_LEVELS+AUX_LEVELS][MAX_MSG_LENGTH+1]

new bool:g_bUnderCS
new g_iMaxPlayers
new g_iMaxStreakLevel, g_iStreakOverIndex, g_iStreakExtraIndex
new g_iPlayerStreakLevel[MAX_CLIENTS+1]
new g_iPlayerStreakKills[MAX_CLIENTS+1]
new g_iPlayerStreakFrags[MAX_CLIENTS+1]
new g_cvarKillsPerStreak
new g_cvarKillsBasis
new g_cvarAnnounceExtra
new g_cvarResetOnNewRound
new g_cvarFFA, g_cvarTeamPlay
new g_cvarLogDetail
//-----------------------------------------------------------------------------
public plugin_precache()
{
	new i

	readConfigFile()
	if(!g_iMaxStreakLevel)
	{   // load built-in preset...
		log_amx("Using built-in preset")

		for(i=0; i<sizeof(g_sDefPreset); i++)
			if(addPreset(g_sDefPreset[i]))
			{
				g_iMaxStreakLevel++
				if(g_iMaxStreakLevel >= MAX_KILLSTREAK_LEVELS + AUX_LEVELS) 
					break
			}		
	}
#if defined _PRECACHE	
	if(g_iMaxStreakLevel)
	{
		for(i=0; i<g_iMaxStreakLevel; i++)
		{
			if(g_sSndStreakKiller[i][0])
				precache_sound(g_sSndStreakKiller[i])
			if(g_sSndStreakOthers[i][0])
				precache_sound(g_sSndStreakOthers[i])
		}
	}
#endif
#if defined _DEBUG	
	log_amx("g_iMaxStreakLevel = %d", g_iMaxStreakLevel)
#endif
	g_iMaxStreakLevel--
	g_iStreakOverIndex = g_iMaxStreakLevel
	g_iMaxStreakLevel--
	g_iStreakExtraIndex = g_iMaxStreakLevel
	
	if(g_iMaxStreakLevel < 0)
		g_iMaxStreakLevel = 0
}
//-----------------------------------------------------------------------------
public plugin_init()
{
	g_bUnderCS = bool:cstrike_running()
	g_iMaxPlayers = get_maxplayers()

	register_plugin(MY_PLUGIN_NAME, MY_PLUGIN_VERSION, MY_PLUGIN_AUTHOR)
	register_cvar("version_preset_killstreak", MY_PLUGIN_VERSION, FCVAR_SERVER|FCVAR_SPONLY)
	
	g_cvarKillsPerStreak = register_cvar("pks_kills_per_streak", "5")
	g_cvarKillsBasis = register_cvar("pks_kills_basis", "0")
	g_cvarAnnounceExtra = register_cvar("pks_announce_extra_levels", "1")

	if(g_bUnderCS)
	{
		g_cvarResetOnNewRound = register_cvar("pks_reset_on_restart", "0")
		register_event("HLTV", "onNewRound", "a", "1=0", "2=0")
	}

	register_event("DeathMsg", "onDeath", "a")
}
//-----------------------------------------------------------------------------
public plugin_cfg()
{
	g_cvarFFA = get_cvar_pointer("mp_freeforall")
	g_cvarTeamPlay = get_cvar_pointer("mp_teamplay")
	g_cvarLogDetail = get_cvar_pointer("mp_logdetail")

	//if(!g_bUnderCS) onNewRound()
}
//-----------------------------------------------------------------------------
public onNewRound()
{
	if(!get_pcvar_num(g_cvarResetOnNewRound))
		return
#if defined _DEBUG
	log_amx("reset streaks")
#endif
	for(new i=1; i<=g_iMaxPlayers; i++)
	{
		g_iPlayerStreakLevel[i] = 0
		g_iPlayerStreakKills[i] = 0
		g_iPlayerStreakFrags[i] = 0
	}
}
//-----------------------------------------------------------------------------
public client_putinserver(id)
{
	g_iPlayerStreakLevel[id] = 0
	g_iPlayerStreakKills[id] = 0
	g_iPlayerStreakFrags[id] = 0
}
//-----------------------------------------------------------------------------
public client_disconnect(id)
{
	g_iPlayerStreakLevel[id] = 0
	g_iPlayerStreakKills[id] = 0
	g_iPlayerStreakFrags[id] = 0
}
//-----------------------------------------------------------------------------
public onDeath()
{
	new bool:bAnnounceExtra = bool:get_pcvar_num(g_cvarAnnounceExtra)
	new iKiller = read_data(1)
	new iVictim = read_data(2)
	new sKiller[32], sVictim[32]
	//new iHeadShot = read_data(3)
	
	if(!iVictim || (!g_iMaxStreakLevel && g_iStreakExtraIndex < 0))
		return

	get_user_name(iVictim, sVictim, sizeof(sVictim)-1)

	if(!iKiller)
		sKiller = "worldspawn"
	else if(iKiller == iVictim)
		sKiller = "himself"
	else
		get_user_name(iKiller, sKiller, sizeof(sKiller)-1)

	if(iKiller && getFragByRules(iKiller, iVictim) > 0.0)
	{
		g_iPlayerStreakKills[iKiller]++
		g_iPlayerStreakFrags[iKiller]++

		new iFragsPerLevel = get_pcvar_num(g_cvarKillsPerStreak)
		if(!g_iPlayerStreakLevel[iKiller] && get_pcvar_num(g_cvarKillsBasis))
			iFragsPerLevel += get_pcvar_num(g_cvarKillsBasis)

		if(g_iPlayerStreakKills[iKiller] >= iFragsPerLevel)
		{
			new iLevel = g_iPlayerStreakLevel[iKiller]
			if(iLevel >= g_iMaxStreakLevel)
				if(bAnnounceExtra)
					iLevel = g_iStreakExtraIndex
				else
					iLevel = -1

			if(iLevel >= 0)
			{
				if(g_sMsgStreak[iLevel][0])
				{
					new iFrags = g_iPlayerStreakFrags[iKiller]

					// 30,50,255
					set_hudmessage(27,102,228, .y=0.28, .holdtime=1.5, .fadeintime=0.1, .fadeouttime=2.0, .channel=-1)
					show_hudmessage(0, g_sMsgStreak[iLevel], sKiller, iFrags)
					console_print(0, g_sMsgStreak[iLevel], sKiller, iFrags)
					if(g_cvarLogDetail && get_pcvar_num(g_cvarLogDetail) > 2 
					&& g_iStreakExtraIndex >= 0)
					{	// put a message into HL1 log
						new sPlayerInfo[128], sAuthId[20], sTeam[10]
					
						get_user_authid(iKiller, sAuthId, sizeof(sAuthId)-1)
						get_user_team(iKiller, sTeam, sizeof(sTeam)-1)
					
						formatex(sPlayerInfo, sizeof(sPlayerInfo)-1, "\"%s<%d><%s><%s>\"", 
							sKiller, get_user_userid(iKiller), sAuthId, sTeam)
					
						log_message(g_sMsgStreak[g_iStreakExtraIndex], sPlayerInfo, iFrags)
					}
				}

				playStreakSounds(iLevel, iKiller, iVictim)
			}
			
			g_iPlayerStreakLevel[iKiller]++
			g_iPlayerStreakKills[iKiller] = 0
		}
	}

	if(g_iPlayerStreakLevel[iVictim] && g_iStreakOverIndex >= 0)
	{
		set_hudmessage(27,102,228, .y=0.31, .holdtime=1.5, .fadeintime=0.1, .fadeouttime=2.0, .channel=-1)
		show_hudmessage(0, g_sMsgStreak[g_iStreakOverIndex], sVictim, sKiller)
		console_print(0, g_sMsgStreak[g_iStreakOverIndex], sVictim, sKiller)

		playStreakSounds(g_iStreakOverIndex, iKiller, iVictim)
	}

	g_iPlayerStreakLevel[iVictim] = 0
	g_iPlayerStreakKills[iVictim] = 0
	g_iPlayerStreakFrags[iVictim] = 0
}
//-----------------------------------------------------------------------------
stock playStreakSounds(level, killer, victim)
{
	if(killer == victim)
		// TODO: play a regret sound
		return

	if(killer && g_sSndStreakKiller[level][0])
		client_cmd(killer, "spk %s", g_sSndStreakKiller[level])
	
	if(g_sSndStreakOthers[level][0])
	{
		new bool:bPlayedOnVictim = false
		new pid, iPlayers[MAX_CLIENTS], i
		get_players(iPlayers, i, "ac") // dead_no|bot_no
		for(--i; i>=0; i--)
		{
			pid = iPlayers[i]
			if(pid != killer)
			{
				client_cmd(pid, "spk %s", g_sSndStreakOthers[level])
				if(pid == victim)
					bPlayedOnVictim = true
			}
		}

		if(!bPlayedOnVictim)
			client_cmd(victim, "spk %s", g_sSndStreakOthers[level])
	}
}
//-----------------------------------------------------------------------------
stock Float:getFragByRules(attacker, victim)
{
	if(attacker == victim)
		return -1.0
	
	if((g_cvarFFA && get_pcvar_num(g_cvarFFA)) || (g_cvarTeamPlay && !get_pcvar_num(g_cvarTeamPlay)))
		return 1.0

	// it's teamplay game
	if(get_user_team(attacker) == get_user_team(victim))
		return -1.0
	
	return 1.0
}
//-----------------------------------------------------------------------------
stock readConfigFile()
{
    new bool:bCustom
    new i, iRetLength
    new sConfigDir[64], sPath[255]
    new sBuffer[MAX_MSG_LENGTH+MAX_SND_LENGTH*2+sizeof(g_sSplitter)*3]
    
    g_iMaxStreakLevel = 0

    get_configsdir(sConfigDir, sizeof(sConfigDir)-1)
    get_mapname(sBuffer, sizeof(sBuffer)-1)

    formatex(sPath, sizeof(sPath)-1, "%s/%s/%s.ini", sConfigDir, g_sAuxConfDir, sBuffer)
	if(!file_exists(sPath)) {
		// get map prefix...
		i = contain(sBuffer, "_")
		if(i >= 0) {
			sBuffer[i+1] = 0
			formatex(sPath, sizeof(sPath)-1, "%s/%s/%s.ini", sConfigDir, g_sAuxConfDir, sBuffer)
			bCustom = bool:file_exists(sPath)
		}

	    if(!bCustom) {
	    	formatex(sPath, sizeof(sPath)-1, "%s/%s", sConfigDir, g_sConfFileName)
	    	if(!fileExists(sPath))
	    	    return
		}
	} else
		bCustom = true
	
	if(bCustom)
		log_amx("Using custom streak preset: \"%s\"", sPath)

    i = 0
    while(read_file(sPath, i, sBuffer, sizeof(sBuffer)-1, iRetLength))
    {
        i++
        // ignore comments & empty strings
        if(sBuffer[0] == ';' || !iRetLength) continue
        if(addPreset(sBuffer))
        {            
            g_iMaxStreakLevel++
            if(g_iMaxStreakLevel >= MAX_KILLSTREAK_LEVELS + AUX_LEVELS) break
        }
    }
	
}
//-----------------------------------------------------------------------------
stock bool:addPreset(const data[])
{
	new sMsg[MAX_MSG_LENGTH+1], sSnd[MAX_SND_LENGTH*2+sizeof(g_sSplitter)]
#if defined _DEBUG
	log_amx("addPreset(data='%s')", data)
#endif
	split(data, sMsg, MAX_MSG_LENGTH, sSnd, sizeof(sSnd)-1, g_sSplitter)
	
	g_sMsgStreak[g_iMaxStreakLevel] = sMsg
	
	new i = contain(sSnd, g_sSplitter)
	if(i < 0)
	{
		copy(g_sSndStreakKiller[g_iMaxStreakLevel], MAX_SND_LENGTH, sSnd)
		g_sSndStreakOthers[g_iMaxStreakLevel][0] = 0
	}
	else
	{
		copy(g_sSndStreakOthers[g_iMaxStreakLevel], MAX_SND_LENGTH, sSnd[i + strlen(g_sSplitter)])
		i = i > MAX_SND_LENGTH ? MAX_SND_LENGTH : i
		copy(g_sSndStreakKiller[g_iMaxStreakLevel], i, sSnd)
	}
	
	if(g_sSndStreakKiller[g_iMaxStreakLevel][0])
	{
		formatex(sSnd, sizeof(sSnd) -1, "%s/%s", g_sSoundDir, g_sSndStreakKiller[g_iMaxStreakLevel])
		if(!fileExists(sSnd))
			g_sSndStreakKiller[g_iMaxStreakLevel][0] = 0
	}
		
	if(g_sSndStreakOthers[g_iMaxStreakLevel][0])
	{
		formatex(sSnd, sizeof(sSnd) -1, "%s/%s", g_sSoundDir, g_sSndStreakOthers[g_iMaxStreakLevel])
		if(!fileExists(sSnd))
			g_sSndStreakOthers[g_iMaxStreakLevel][0] = 0
	}
#if defined _DEBUG
	log_amx("g_sSndStreakKiller[%d] = '%s'", g_iMaxStreakLevel, g_sSndStreakKiller[g_iMaxStreakLevel])
	log_amx("g_sSndStreakOthers[%d] = '%s'", g_iMaxStreakLevel, g_sSndStreakOthers[g_iMaxStreakLevel])
#endif

	return true
}
//-----------------------------------------------------------------------------
stock bool:fileExists(const filename[])
{
	if(!file_exists(filename))
	{
		log_amx("File not found: \"%s\"", filename)
		return false
	}

	return true
}
//-----------------------------------------------------------------------------
